<?php 
namespace WOO_PRODUCT_TABLE_PRO\Inc\Features;

/**
 * We will call it, when only ajax call is run.
 * 
 * Actually acf's shortcode [acf field='123' post_id='2323'] is not workifng on ajax
 * when not login user,
 * I have updated this
 * 
 * and regenerate it.
 * 
 * @author Saiful Islam <codersaiful@gmail.com>
 * @package Woo Product Table
 * 
 * @since 8.1.1.0 
 */
class ACF_Shortcode_Render{

    public function run(){
        if(! defined('DOING_AJAX')) return;

        add_shortcode('acf', [$this, 'acf_shortcode']);
        
    }


    /**
     * We have regenerate ACF shortcode
     * Actually a customer faced a issue that acf field's shortcode was not working when loading on ajax.
     *
     * @param [type] $atts
     * @return void
     */
    public function acf_shortcode($atts){

        $keyword = $atts['field'] ?? '';
        $id = $atts['post_id'] ?? get_the_ID();
        // var_dump(get_the_ID());
        if( empty($id) || empty($keyword) ){
            echo "field and post_id attr required";
            return;
        }
        $acf_obj = get_field_object($keyword, $id);
        

        /**
         * This is for Critical Situation, When Product import using any Import plugin and acf get_field_object() dont work
         * acf value wan't showing after import from another site
         * @author Fazle Bari 
         * @author Saiful Islam <codersaiful@gmail.com>
         * @Note Actually we get this error for a client, then we realized that issue.
         */
        if( ! $acf_obj ){
            $acf_val = get_field( $keyword, $id );
            echo esc_html( $acf_val );
            return;
        }

        
        $get_field = get_field($keyword, $id);
        $type = isset( $acf_obj['type'] ) ? $acf_obj['type'] : false;
        if( !$type ){
            return;
        }
        global $product;
        /**
         * Actually I get it from acf field, where all is working propertly
         */
        $label_text = $column_settings = $settings = $table_ID = false;
        $target = apply_filters( 'wpto_acf_link_target', '_blank', $settings, $column_settings, $keyword, $product, $table_ID );
        ob_start();
        switch ($type){
            case 'file':
                $value = isset( $acf_obj['value'] ) ? $acf_obj['value'] : false;
                if( $value && isset( $value['url'] ) && !empty( $value['url'] )){
                $url =  $value['url'];   
                $label = $label_text ? $label_text :  $acf_obj['label'];
                $label = apply_filters( 'wpto_extra_label_text', $label, $id, $keyword, $acf_obj, $settings );
                ?>
        <a class="wpt_acf wpt_type_file" target="<?php echo esc_attr( $target ); ?>" href="<?php echo esc_url( $url ) ?>"><?php echo esc_html( $label ); ?></a>
                <?php
                }
                break;
            
            case 'url':
                $value = isset( $acf_obj['value'] ) ? $acf_obj['value'] : false;
                if( !empty( $value ) ){
                $label = $label_text ? $label_text :  $acf_obj['label'];
                $label = apply_filters( 'wpto_extra_label_text', $label, $id, $keyword, $acf_obj, $settings );
                ?>
        <a class="wpt_acf wpt_type_url" target="_blank" href="<?php echo esc_url( $value ) ?>"><?php echo esc_html( $label ); ?></a>
                <?php
                }
                break;
            
            case 'email':
                $value = isset( $acf_obj['value'] ) ? $acf_obj['value'] : false;
                if( !empty( $value ) ){
                $label = $label_text ? $label_text :  $value;
                ?>
        <a class="wpt_acf wpt_type_email"  target="<?php echo esc_attr( $target ); ?>" href="mailto:<?php echo esc_attr( $value ) ?>"><?php echo esc_html( $label ); ?></a>
                <?php
                }
                break;
            
            case 'select':
            case 'radio':
                $value = isset( $acf_obj['value'] ) ? $acf_obj['value'] : false;
                echo isset( $acf_obj['choices'][$value] ) ? $acf_obj['choices'][$value] : false;
                break;
            
            case 'image':
                $target_url = false;
                $img_target_url_keyword = apply_filters( 'wpto_image_url_keyword','image_url', $settings, $column_settings, $keyword, $product, $table_ID );
                $target_url_acf_obj = get_field_object( $img_target_url_keyword, $id);
                if($target_url_acf_obj){
                    $url = isset( $target_url_acf_obj['value'] ) ? $target_url_acf_obj['value'] : false;
                    $target_url = !empty( $url ) && is_string( $url ) ? $url : false;
                }
                $target_url = apply_filters( 'wpto_image_target_url', $target_url, $product, $settings, $column_settings, $keyword, $table_ID);
                if( $target_url ){
                ?>
                <a href="<?php echo esc_attr( $target_url ); ?>"  target="<?php echo esc_attr( $target ); ?>">     
                <?php
                }
                
                $value = isset( $acf_obj['value'] ) ? $acf_obj['value'] : false;
                if( $value && isset( $value['url'] ) && !empty( $value['url'] ) ){
                $url =  $value['url'];   
                ?>
                <img class="wpt_acf wpt_type_file" src="<?php echo esc_url( $url ) ?>">
                
                <?php
                }
                
                if( $target_url ){
                ?>
                <a href="<?php echo esc_attr( $target_url ); ?>"  target="<?php echo esc_attr( $target ); ?>">     
                <?php
                }
                
                break;
                
            /**
             * Helped taken from Bari
             * 
             * @since 7.0.5.1
             */
            case 'checkbox':
                
                $values = isset( $acf_obj['value'] ) ? $acf_obj['value'] : false;

                if( is_array( $values ) ){
                    echo esc_html( implode( ', ', $values ) );
                }
                break;
            
            default:
                echo is_string( $get_field ) ? $get_field : false;
        }
        return ob_get_clean();

    }
}