<?php
/****** PRO VERSION - INCLUDE / FRONTEND FUNCTION *******
 * WPT PRO VERSION
 * MAIN MODULE
 */

use function WPML\FP\apply;

$config = get_option( 'wpt_configure_options' ); 

add_filter( 'wpto_item_final_loc', 'wpt_pro_template_for_guest_user', 10, 6 );

if( !function_exists( 'wpt_pro_template_for_guest_user' ) ){
    /**
     * Chnage the template url for not logged in user
     * 
     * @since 7.0.9.0
     */
    function wpt_pro_template_for_guest_user( $file, $file_name, $items_directory_2, $keyword, $table_ID, $settings ){
        if( is_user_logged_in() ){
            return $file;
        }
        
        $only_log_file = false;
        if( isset( $settings['only_login_user'] ) ){
            $only_log_file = WPTP_BASE_DIR . 'templates/only-login-user.php';
        }
        if( is_file( $only_log_file ) ){
            $file = $only_log_file;
        }
        return $file;
    }
}

if( !function_exists( 'wpt_pro_user_login_col' ) ){
    /**
     * To hide colum based on LogedIn user 
     * getting data from post data
     * Used: $enabled_column_array = apply_filters( 'wpto_enabled_column_array', $enabled_column_array, $table_ID, $atts, $column_settings, $column_array ); 
     * 
     * @param type $enabled_column_array
     * @param type $table_ID
     * @param type $atts
     * @param type $column_settings
     * @param type $column_array
     * @return type
     */
    function wpt_pro_user_login_col( $enabled_column_array, $table_ID, $atts, $column_settings  ){
        if( !is_array( $enabled_column_array ) || is_user_logged_in() ){
            return $enabled_column_array;
        }
        $configure = get_option('wpt_configure_options');
        
        foreach( $enabled_column_array as $key=>$colname ){
            if( isset( $column_settings[$key]['only_login_user'] ) && ! isset( $configure['column_hide_for_guest'] ) ){
                unset( $enabled_column_array[$key] );
            }
        }
        return $enabled_column_array;
    }
}
add_filter( 'wpto_enabled_column_array', 'wpt_pro_user_login_col', 10, 4 );

if( !function_exists( 'wpt_pro_user_login_item' ) ){
    
    /**
     * To hide Item of inside colum based on LogedIn user 
     * getting data from post data
     * 
     * $extra_items = apply_filters( 'wpto_extra_items_arr', $extra_items, $keyword, $column_settings, $table_ID, $product );
     * @param type $settings
     * @param type $table_ID
     * @param type $product
     * @param type $column_settings
     * @return type
     */
    function wpt_pro_user_login_item( $extra_items, $keyword, $column_settings ){
        if( !is_array( $extra_items ) || is_user_logged_in() ){
            return $extra_items;
        }
        $configure = get_option('wpt_configure_options');
        foreach( $extra_items as $i=>$item ){
            if( isset( $column_settings[$item]['only_login_user'] ) && ! isset( $configure['column_hide_for_guest'] ) ){
                // unset( $extra_items[$i] );
            }
        }
        return $extra_items;
    }
}
add_filter( 'wpto_extra_items_arr', 'wpt_pro_user_login_item', 10, 3 );

if( !function_exists( 'wpt_pro_add_default_col_arr' ) ){
    /**
     * Added two new Default Column for Pro version
     * 
     * @param type $column_array
     */
    function wpt_pro_add_default_col_arr( $column_array ){
        $column_array['shortcode'] = __( 'Shortcode', 'wpt_pro' );
        $column_array['content'] = __( 'Content', 'wpt_pro' );
        $column_array['audio_player'] = __( 'Audio', 'wpt_pro' );
        $column_array['quick_qty'] = __( 'Quick Qty', 'wpt_pro' ); //Added at V7.0.9.0
        $column_array['variation_name'] = __( 'Variation Names', 'wpt_pro' );//8.0.3.1
        $column_array['viewed'] = __( 'Viewed', 'wpt_pro' ); //Added at V78.0.1.0
        $column_array['toggle_description'] = __( 'Toggle Description', 'wpt_pro' ); //Added at V8.0.2.2
        return $column_array;
    }
}
add_filter( 'wpto_default_column_arr', 'wpt_pro_add_default_col_arr' );

if( !function_exists( 'wpt_pro_change_template_folder' ) ){
    
    /**
     * TEMPLATE MAIN FOLDER DIRECTORY change for our Special column content
     * currently we have added new only two new column
     * and add two items file inside includes/items/
     * And we need to change main directory for that, and we did it
     * Even user also able to change template file loc
     * 
     * NEW ADDED: TEMPLATE CHANGE on type wise. Column Type Wise.
     * usings filter: $items_directory_1 = apply_filters('wpto_template_folder', $items_directory,$keyword, $type, $table_ID, $product, $settings, $column_settings );
     * 
     * 
     * @param String $items_directory
     * @param String $keyword
     * @return String
     */
    function wpt_pro_change_template_folder( $items_directory, $keyword, $type ){
        $our_pro_column = array(
            'shortcode',
            'content',
            'audio_player',
            'quick_qty', //Added at V7.0.9.0
            'variation_name', //Added at V8.0.3.1
            'viewed', //Added at V8.0.1.0
            'toggle_description', //Added at V8.0.1.0
        );
        
        $our_pro_type = array(
            //'blank_type',
            'acf',
            'audio',
            'hook_action',
            'array',
            'all_content',
            'single_variation'
        );
        $items_pro_dir = WPTP_DIR_BASE . 'includes/items/';
        //var_dump(in_array( $keyword, $our_pro_column ),in_array( $keyword, $our_pro_column ) ? $items_permanent_dir : $items_directory );
        $my_item_dir = in_array( $keyword, $our_pro_column ) ||  in_array( $type, $our_pro_type ) ? $items_pro_dir : $items_directory;
        return $my_item_dir;
    }
}
add_filter( 'wpto_template_folder', 'wpt_pro_change_template_folder', 50, 3 );


if( !function_exists( 'wpt_pro_args_manipulation_frontend' ) ){
    /**
     * IN PRO
     * Args Manipulation for FrontEnd
     * By Useing following Filter
     * $args = apply_filters( 'wpto_table_query_args', $args, $table_ID, $atts, $column_settings, $enabled_column_array, $column_array );
     * 
     * @param type $args
     * @param type $table_ID
     * @param type $atts
     * @param type $column_settings
     * @param type $enabled_column_array
     * @param type $column_array
     * @return type
     */
    function wpt_pro_args_manipulation_frontend( $args, $table_ID, $atts, $column_settings, $enabled_column_array, $column_array ){
        
        /**
         * Has Transferred into Free Version
        //MainTain for Archives Page
        global $wpdb;
        $page_query = isset( $GLOBALS['wp_query'] ) ? $GLOBALS['wp_query']->query_vars : null;
        $args_product_in = false;
        if( isset( $page_query['wc_query'] ) && $page_query['wc_query'] == 'product_query' ){
            $gen_args = array_merge( $args,$GLOBALS['wp_query']->query_vars );
            $gen_args['post_type'] = isset( $args['post_type'] ) && !empty( $args['post_type'] ) ? $args['post_type'] : 'product';
            $args = $gen_args;

            $sql = $GLOBALS['wp_query']->request;
            $results = $wpdb->get_results( $sql, ARRAY_A );
            $args_product_in = array();
            foreach( $results as $result ){
                $args_product_in[] = $result['ID'];
            }
            $args['post__in'] = $args_product_in;
            $args['paged'] = 0;
            unset( $args['tax_query'] );
            unset( $args['meta_query'] );
        }
        */
        
        
        /**
         * Args Management from atts
         */
        $temp_atts = $atts;
        unset($temp_atts['id']);
        unset($temp_atts['name']);
        unset($temp_atts['meta_query']);
        unset($temp_atts['tax_query']);
        unset($temp_atts['table_ID']);
        if( is_array( $temp_atts ) && count( $temp_atts ) > 0 ){
            foreach( $temp_atts as $att_key => $att_val ){
               $args[$att_key] = $att_val;
            }
        }
        $ids = $temp_atts['ids'] ?? false;
        if(is_string( $ids ) && ! empty( $ids ) ){
            $args['post__in'] = wpt_explode_string_to_array($ids);
        }
        return $args;
    }
}
add_filter( 'wpto_table_query_args', 'wpt_pro_args_manipulation_frontend', 10, 6 );

if( !function_exists( 'wpt_shortcode_column_add' ) ){
    /**
     * Add new shortcode Column for Product Table Edit part
     * to show new column in table
     * 
     * @param Array $column_settings
     */
    function wpt_shortcode_column_add( $_device_name, $column_settings ){
        
       $shortcode = isset( $column_settings['shortcode']['content'] ) ? $column_settings['shortcode']['content'] : false;
       ?>
        <label>Input Your Shortcode Content:</label>
        <input type="text" name="column_settings<?php echo $_device_name; ?>[shortcode][content]" value='<?php echo esc_attr( $shortcode ); ?>'>
        <div class="wpt-column-info">
            <b>Supported Dynamic Value</b>
            <ul>
                <li><code>%product_id%</code>: Product ID</li>
                <li><code>%sku%</code>: Product SKU</li>
                <li><code>%name%</code>: Product Title</li>
                <li><code>%excerpt%</code>: Short Description</li>
                <li>... <a href="https://wooproducttable.com/supported-attributes-of-shortcode-and-content-column/" target="_blank"><?php echo esc_html__( 'Full list of Dynamic Shortcode', 'wpt_pro' ); ?></a></li>
            </ul>
            <p class="hints">Example: <strong>[favorite_button post_id="%product_id%" title="%name%"]</strong><br /></p>
        </div>
       <?php
   }
}
add_action( 'wpto_column_setting_form_shortcode', 'wpt_shortcode_column_add', 10, 2 );

if( !function_exists( 'wpt_content_column_add' ) ){
    
    
   function wpt_content_column_add( $_device_name, $column_settings ){

        $my_values = isset( $column_settings['content']['content'] ) ? $column_settings['content']['content'] : false;

        $settings = array(
            'textarea_name'     =>'column_settings' . $_device_name . '[content][content]',
            'textarea_rows'     => 3,
            'teeny'             => false,
            'autocomplete'             => 'on',
            );
        wp_editor( wp_kses_post( $my_values ), 'wpt_content_column' . $_device_name, $settings ); 
        ?>
        <div class="wpt-column-info">
            <b>Supported Dynamic Value</b>
            <ul>
                <li><code>%product_id%</code>: Product ID</li>
                <li><code>%sku%</code>: Product SKU</li>
                <li><code>%name%</code>: Product Title</li>
                <li><code>%excerpt%</code>: Short Description</li>
                <li>... <a href="https://wooproducttable.com/supported-attributes-of-shortcode-and-content-column/" target="_blank"><?php echo esc_html__( 'Full list of Dynamic Shortcode', 'wpt_pro' ); ?></a></li>
            </ul>
            <p class="hints"> <?php echo esc_html__( 'Example of Content Box with Supported Shortcode', 'wpt_pro' ); ?> <a href="https://wooproducttable.com/supported-attributes-of-shortcode-and-content-column/" target="_blank"><?php echo esc_html__( 'Click Here', 'wpt_pro' ); ?></a></p>
        </div>    
        <?php
    }

}
add_action( 'wpto_column_setting_form_content', 'wpt_content_column_add', 10, 2 );

if( !function_exists( 'wpt_mce_autosave_mod' ) ){
    /**
     * For Testing, It was added, Currently it's disabled.
     * Can be use later
     * 
     * @param array $init
     * @return string
     */
    function wpt_mce_autosave_mod( $init ) {
        $init['setup'] = 'function(a){a.on("change",function(b){jQuery(this).parent().find(\'#btnEditCharJournal\').trigger("click"),console.log("the event object ",b),console.log("the editor object ",a),console.log("the content ",a.getContent())})}';     
        return $init;
    }
}
//add_filter('tiny_mce_before_init', 'wpt_mce_autosave_mod');




if( !function_exists( 'wpt_add_extra_inside_items' ) ){
    function wpt_add_extra_inside_items( $columns_array ){
        //wpto_inside_item_arr
        $columns_array['menu_order'] = "Menu Order";
        //$columns_array['menu_order'] = "Menu Order";
        return $columns_array;
    }
}

add_filter( 'wpto_inside_item_arr', 'wpt_add_extra_inside_items' ); //$items,$keyword, $column_settings, $columns_array, $post

if( !function_exists( 'wpt_pro_body_class' ) ){
    /**
     * Special Boddy Class for Pro Version
     * 
     * @global type $post
     * @global type $shortCodeText
     * @param type $class
     * @return string
     */
    function wpt_pro_body_class( $class ) {
        global $post,$shortCodeText;

        if( isset($post->post_content) && has_shortcode( $post->post_content, $shortCodeText ) ) {
            $class[] = 'wpt_table_body_pro';
            $class[] = 'wpt_pro_table_body';
        }
        return $class;
    }
}
add_filter( 'body_class', 'wpt_pro_body_class' );

if( !function_exists( 'wpt_custom_search_join' ) ){

    /**
     * Extend WordPress search to include custom fields
     *
     * Join posts and postmeta tables
     *
     * http://codex.wordpress.org/Plugin_API/Filter_Reference/posts_join
     */
    function wpt_custom_search_join( $join,$wp_query ) {

        $validation = $wp_query->get( 'wpt_query_type' );//Value to be search, otherwise, return $where
        if( $validation !== 'search' ) return $join;
        
        $wpt_join = $wp_query->get( 'wpt_join' );
        if( $wpt_join !== true ) return $join;

        global $wpdb;

        // $config = get_option( 'wpt_configure_options' ); //advance_search
        // $v_fix = isset( $config['advance_search_issue'] ) && $config['advance_search_issue'] == 'on' ? false : true;
        
        $v_fix = apply_filters('wpt_adv_search_fixer', true);

        if( $v_fix ){
            $join .=' INNER JOIN '.$wpdb->postmeta. ' AS wpt_table ON '. $wpdb->posts . '.ID = wpt_table.post_id ';
        }

        return $join;
    }
}

if( !function_exists( 'wpt_custom_search_where' ) ){
    /**
     * Modify the search query with posts_where
     *
     * http://codex.wordpress.org/Plugin_API/Filter_Reference/posts_where
     */
    function wpt_custom_search_where( $where, $wp_query ) {

        $validation = $wp_query->get( 'wpt_query_type' );//Value to be search, otherwise, return $where
        if( $validation !== 'search' ) return $where;
        
        if( empty( $wp_query->query_vars['wpt-s'] ) ) return $where;
        global $wpdb;

        $search_from = $wp_query->query_vars['search_from'];

        $search_term = $wp_query->query_vars['wpt-s']; //Already checked before this line. Code: if( empty( $wp_query->query_vars['wpt-s'] ) ) return $where;
        $search_term = trim($search_term);

        if ( $search_from && ! empty( $search_term ) ) {
            $raw_sql = $wp_query->get( 'wpt_raw_sql' );
            $compare = $wp_query->get( 'wpt_compare' );

            $inter_sql = array();
            foreach( $search_from as $search_item => $table_name ){

                // $numeric = is_numeric( $search_term );
                // $percent = ! $numeric ? "%" : "";
                $percent = $compare == 'on' ? '' : "%";

                if( $table_name == 'posts' ){

                    $sql_posts = $wpdb->posts . ".". $search_item . ' LIKE \'' . $percent . esc_sql( like_escape( $search_term ) ) . $percent . '\'';

                    $inter_sql[] = $sql_posts;
                }
                /**
                 * Why did we use $raw_sql or 'wpt_raw_sql',
                 * Actually If a user select searchFrom only from post_meta
                 * We will query by WordPres defatul query system.
                 * such:
                 * $args['meta_query'] = [];
                 * 
                 * Othewise, I will use sql query 
                 */
                elseif( $table_name == 'postmeta' && $raw_sql ){
                    $inter_sql[] = "(wpt_table.meta_key='{$search_item}' AND wpt_table.meta_value LIKE'" . $percent . esc_sql( like_escape( $search_term ) ) . $percent . "')";
                }
            }

            $inter_sql_implode = implode( ' OR ', $inter_sql );
            $where .= ! empty( $inter_sql ) ? ' AND (' . $inter_sql_implode . ')' : '';
        }else{
            $myPreg = "(".$wpdb->posts.".post_title LIKE $1) OR (wpt_table.meta_value LIKE $1" . ")";
            $where = preg_replace("/\(\s*".$wpdb->posts.".post_title\s+LIKE\s*(\'[^\']+\')\s*\)/",$myPreg, $where );

        }


        return $where;
    }
}

if( !function_exists( 'wpt_custom_search_distinct' ) ){
    /**
     * Prevent duplicates
     *
     * http://codex.wordpress.org/Plugin_API/Filter_Reference/posts_distinct
     */
    function wpt_custom_search_distinct( $where, $wp_query  ) {
        
        $validation = $wp_query->get( 'wpt_query_type' );//Value to be search, otherwise, return $where
        if( $validation !== 'search' ) return $where;
        
        $wpt_join = $wp_query->get( 'wpt_join' );
        if( $wpt_join !== true ) return $where;

            global $wpdb;
            return "DISTINCT";
            return $where;

    }
}
//add_filter('posts_request', function( $request, $wp_query){
//    print_r($request);
//    return $request;
//}, 10, 2);
if( !function_exists( 'wpt_search_filter' ) ){
    function wpt_search_filter(){

        /**
         * If any user fance any issue for advance search,
         * just use
         * following filter like:
         * add_filter( 'wpto_advance_search_bool', '__return_false' );
         * 
         * @Hook wpto_advance_search_bool
         * @since 3.1.8.4
         * @author Saiful Islam <codersaiful@gmail.com>
         */
        $advance_search = apply_filters( 'wpto_advance_search_bool', true );
        if( ! $advance_search ) return;
        add_filter('posts_join', 'wpt_custom_search_join', 10, 2 );
        add_filter( 'posts_where', 'wpt_custom_search_where',10 ,2 );
        add_filter( 'posts_distinct', 'wpt_custom_search_distinct', 10, 2 );

    }
}

// $config = get_option( 'wpt_configure_options' ); //advance_search
//  if( isset( $config['advance_search'] ) && $config['advance_search'] == 'on' ){
//     add_filter('init','wpt_search_filter');
//  }

/**
 * Advance search feature is enable by default
 * that's why, commented up code 
 * @since 3.1.8.4
 */
add_filter('init','wpt_search_filter');

if( !function_exists( 'wpt_taxonomy_page_template' ) ){
   
    function wpt_taxonomy_page_template( $template_file ) {
        
        //Taxonomy page na hole amra niche jaboi na.
        if( ! is_product_taxonomy() ) return $template_file;

        $id = get_queried_object_id();
        $taxo_table_id = get_term_meta( $id, 'table_id', true );
        
        //empty hole ba numeric na hole niche jaboi na.
        if( empty( $taxo_table_id ) || !is_numeric( $taxo_table_id ) ) return $template_file;
        
           
        $my_archive = WPTP_BASE_DIR . 'templates/product-archives.php';
        $my_archive = apply_filters( 'wpto_taxonomy_page_template_loc', $my_archive, $template_file );
        $template_file = file_exists( $my_archive ) ? $my_archive : $template_file;

        return $template_file;
    }
}
 
add_filter( 'template_include', 'wpt_taxonomy_page_template', 99999 );




if( !function_exists( 'wpt_archives_page_template' ) ){
   
    function wpt_archives_page_template( $template_file ) {
        
        
        /**
         * Detect Search Page
         * Actually when user will search product using wc default search box
         */
        $search_page = isset( $_GET['s'] ) && isset( $_GET['post_type'] ) && $_GET['post_type'] == 'product' ? true : false;
        
        $search_page = apply_filters('wpto_search_page_template', $search_page, $template_file );
        
        $shop_display = get_option('woocommerce_shop_page_display');
        $cat_display = get_option('woocommerce_category_archive_display');

        $id = get_queried_object_id();

        
        $child = get_term_children( $id, 'product_cat');    
        
        if( ( is_shop() && $shop_display == 'subcategories' && ! $search_page ) || ( ! empty( $child ) && $cat_display == 'subcategories' && ! $search_page ) ){
            return $template_file;
        }
        $page_template_validation = apply_filters( 'wpto_direct_template_validation', true, $template_file );
        
        if( ( is_shop() || is_product_taxonomy() ) && $page_template_validation ){
           
           $my_archive = WPTP_BASE_DIR . 'templates/product-archives.php';
           $my_archive = apply_filters( 'wpto_archvie_page_template_loc', $my_archive, $template_file );
           return file_exists( $my_archive ) ? $my_archive : $template_file;
       }
       
       if( $page_template_validation && strpos( $template_file, 'dokan' ) > 0 && strpos( $template_file, 'store' ) > 0 ){
           $my_archive = WPTP_BASE_DIR . 'templates/store.php';
           $my_archive = apply_filters( 'wpto_dokan_page_template_loc', $my_archive, $template_file );
           return file_exists( $my_archive ) ? $my_archive : $template_file;
       }
       
       return $template_file;
    }
}
if( isset( $config['table_on_archive'] ) && $config['table_on_archive'] == 'on' ){  
    add_filter( 'template_include', 'wpt_archives_page_template', 999 );
}

if( !function_exists( 'wpt_playlist_class_body' ) ){
     
    function wpt_playlist_class_body( $table_class_arr ){
        $table_class_arr[] = 'playlist';
        return $table_class_arr;
    }

}
add_filter( 'wpto_table_tag_class_arr', 'wpt_playlist_class_body' );

//add_action( 'wpto_column_setting_form_inside_audio_player','wpt_audio_column_settings', 10, 2 );
add_action( 'wpto_column_setting_form_audio_player','wpt_audio_column_settings', 10, 2 );
function wpt_audio_column_settings( $_device_name, $column_settings ){
//     var_dump($column_settings['audio_player']);
    $audio_player = isset( $column_settings['audio_player'] ) ? $column_settings['audio_player'] : false;
    $audio = isset( $audio_player['player_position'] ) ? 'checked' : '';
    $auto_play = isset( $audio_player['auto_play'] ) ? 'checked' : '';
    $b_color = isset( $audio_player['bg_color'] ) ? $audio_player['bg_color'] : '';
    $pbar_color = isset( $audio_player['progressbar_color'] ) ? $audio_player['progressbar_color'] : '';
    
    
    ?>
<ul class="sss">
    <li>
        <label>Stick to Bottom?</label>
        <input type="checkbox" name="column_settings<?php echo $_device_name; ?>[audio_player][player_position]" <?php echo $audio; ?>>
        <!--
        <label>Auto Play</label>
        <input type="checkbox" name="column_settings[audio_player][auto_play]" <?php echo $auto_play; ?>>
        -->
    </li>
    <li>
        <label>Background Color</label>
        <input type="input" class="ua_input wpt-color" name="column_settings<?php echo $_device_name; ?>[audio_player][bg_color]" value="<?php echo $b_color; ?>">
        <label>Progress Bar Color</label>
        <input type="input" class="ua_input wpt-color" name="column_settings<?php echo $_device_name; ?>[audio_player][progressbar_color]" value="<?php echo $pbar_color; ?>">
    </li>
    <li>

<p>
    Create custom field by ACF<br>
    with following name, to display your desired item.<br>
    <code>audio_file</code>: For Audio Source as File Type<br>
    <code>audio_artist</code>: For Artist of Audio, use audio_artist custom field<br>
    <code>audio_title</code>: For Label/Title of Audio, use audio_title custom field. If not found <i>audio_title</i> field, Product tile will display<br>
</p>
<?php
}

/**
 * Toggle Description Column Settings
 * 
 * @since 8.0.2.3
 * @author Md Rakibul Islam
 */
add_action( 'wpto_column_setting_form_inside_toggle_description', 'wpt_toggle_description_column_settings', 10, 2 );
function wpt_toggle_description_column_settings($_device_name, $column_settings){
    $toggle_description = isset( $column_settings['toggle_description'] ) ? $column_settings['toggle_description'] : false;
    $toggle_open_text = isset( $toggle_description['open_text'] ) ? $toggle_description['open_text'] : '';
    $toggle_close_text = isset( $toggle_description['close_text'] ) ? $toggle_description['close_text'] : '';
    ?>
    <ul class="sss">
        <li>
            <label>Toggle Open Text</label>
            <input type="text" class="ua_input" name="column_settings<?php echo esc_attr( $_device_name ); ?>[toggle_description][open_text]" value="<?php echo esc_attr( $toggle_open_text ); ?>">
        </li>
        <li>
            <label>Toggle Close Text</label>
            <input type="text" class="ua_input" name="column_settings<?php echo esc_attr( $_device_name ); ?>[toggle_description][close_text]" value="<?php echo esc_attr( $toggle_close_text ); ?>">
        </li>
    </ul>
    <?php
}
/**
 * Thumbnail Gallery Hook
 *
 * @since 8.0.2.4
 * @author Md Rakibul Islam
 */
if( !function_exists( 'wpt_thumbnail_gallery' ) ) {
    function wpt_thumbnail_gallery( $table_ID, $settings, $column_settings, $config_value, $product)
    {
        $thumb_gallery = isset($column_settings['thumbnails']['thumb_gallery']) ? 'yes' : 'no';
        $thumb_size = isset($config_value['thumbs_image_size']) ? $config_value['thumbs_image_size'] : '150';
        $attachment_ids = $product->get_gallery_image_ids();
        if ($thumb_gallery == 'yes' && !empty($attachment_ids)){
            wp_enqueue_script('flexslider');
            ?>
            <style>
                .wpt_gallery_thumbnails > div > .flex-viewport{
                    max-width: <?php echo esc_attr($thumb_size);?>px;
                }
            </style>
            <div class="wpt_gallery_thumbnails">
                <div id="wpt_gallery_<?php echo esc_attr($product->get_id()); ?>" class="flexslider main-slider-wrapper">
                    <ul class="slides main-slider">
                        <?php
                        if ($attachment_ids && $product->get_image_id()) {
                            ?>
                            <li><?php echo wp_get_attachment_image($product->get_image_id(),array($thumb_size,$thumb_size));?></li>
                            <?php
                            foreach ($attachment_ids as $attachment_id) {
                                ?>
                                <li>
                                    <?php echo wp_get_attachment_image($attachment_id,array($thumb_size,$thumb_size)) ?>
                                </li>
                                <?php
                            }
                        }
                        ?>
                    </ul>
                </div>
                <div id="wpt_thumb_<?php echo esc_attr($product->get_id()); ?>" class="flexslider thumb-slider-wrapper">
                <ul class="slides thumbs-slider">
                    <?php
                    if ($attachment_ids && $product->get_image_id()) {
                        ?>
                        <li><?php echo wp_get_attachment_image($product->get_image_id(), array(500,500));?></li>
                        <?php
                        foreach ($attachment_ids as $attachment_id) {
                            ?>
                            <li>
                                <?php echo wp_get_attachment_image($attachment_id, array(500,500)) ?>
                            </li>
                            <?php
                        }
                    }
                    ?>
                </ul>
            </div>
            </div>
        <?php }
    }
}
add_action( 'wpt_thumbnail_col_bottom', 'wpt_thumbnail_gallery', 10, 5 );

/**
 * Thumbnail Gallery Column Settings
 *
 * @since 8.0.2.4
 * @author Md Rakibul Islam
 */
add_action( 'wpto_column_setting_form_thumbnails', 'wpt_thumbnails_column_gallery', 10, 2 );
function wpt_thumbnails_column_gallery($_device_name, $column_settings){
    $thumb_gallery = isset( $column_settings['thumbnails']['thumb_gallery']) ? true : false;
    $thumb_gallery_checked = $thumb_gallery == true ? 'checked' : '';
    ?>
    <div class="thumb_gallery_wrapper">
        <label for="thumb_gallery<?php echo esc_attr( $_device_name ); ?>">
        <input id="thumb_gallery<?php echo esc_attr( $_device_name ); ?>" title="Enable Gallery On Thumbnail Column" name="column_settings<?php echo esc_attr( $_device_name ); ?>[thumbnails][thumb_gallery]" id="thumb_gallery" class="thumb_gallery" type="checkbox" value="off" <?php echo esc_attr( $thumb_gallery_checked ); ?>>
        Enable Gallery
        </label>
    </div>
    <?php
}

if( !function_exists( 'wpt_title_badge' ) ) {
	
    function wpt_title_badge($table_ID, $settings, $column_settings, $config_value, $product)
    {
        if ( ! $product || ! is_object( $product ) ) {
            return;
        }
        $badge_config = $config_value['wpt_show_badge'] ?? 'off';
        if($badge_config=='on'){
            $sale         = $product->is_on_sale();
            $price_sale   = $product->get_sale_price();
            $price        = $product->get_regular_price();
            $simple       = $product->is_type( 'simple' );
            $variable     = $product->is_type( 'variable' );
            $external     = $product->is_type( 'external' );
            $bundle       = $product->is_type( 'bundle' );
            $sale_text    = 'Sale!';
            $sale_percent = true;
            $final_price  = '';
            $out_of_stock = get_post_meta( $product->get_id(), '_stock_status', true );

            if ( 'outofstock' === $out_of_stock ) {
                return;
            }

            if ( $sale ) {
                // For simple product.
                if ( $simple || $external || $bundle ) {
                    if ( $sale_percent ) {
                        $final_price = ( ( $price - $price_sale ) / $price ) * 100;
                        $final_price = '-' . round( $final_price ) . '%';
                    } elseif ( $sale_text ) {
                        $final_price = $sale_text;
                    }
                } elseif ( $variable && $sale_text ) {
                    // For variable product.
                    $final_price = $sale_text;
                }

                if ( ! $final_price ) {
                    return;
                }

                $final_price = apply_filters( 'wpt_pro_price_flash', $final_price, $product );

                $classes[] = 'wpt-pro-tag-on-sale';
                ?>
                <span class="<?php echo esc_attr( implode( ' ', array_filter( $classes ) ) ); ?>">
                        <?php echo wp_kses_post( $final_price ); ?>
                    </span>
                <?php
            }
        }
    }
}
add_action( 'wpt_title_col_bottom', 'wpt_title_badge', 10, 5 );

if( !function_exists( 'wpt_pro_add_toggle_content' ) ){
    
    /**
     * Showing Toggle Button with text to Show/Hide full content of that TD
     * 
     * @param type $keyword
     * @param type $table_ID
     * @param type $settings Available full setting Like following
     * array (size=8)
  'type' => string 'default' (length=7)
  'type_name' => string 'Default' (length=7)
  'tag' => string '' (length=0)
  'tag_class' => string 'item_product_title_919 item_dJLoDtTFHw' (length=38)
  'toggle' => string 'on' (length=2)
  'toggle_label' => string 'Hello To Bangladesh' (length=19)
  'style' => 
    array (size=11)
      'color' => string '' (length=0)
      'background-color' => string '' (length=0)
      'border' => string '' (length=0)
      'text-align' => string '' (length=0)
      'vertical-align' => string '' (length=0)
      'width' => string '' (length=0)
      'height' => string '' (length=0)
      'font-size' => string '' (length=0)
      'font-style' => string '' (length=0)
      'padding' => string '' (length=0)
      'margin' => string '' (length=0)
  'style_str' => string '' (length=0)
     * 
     * 
     * @return void Return Nothing, It will show a Toggle Button
     */
    function wpt_pro_add_toggle_content( $keyword, $table_ID, $settings ){
        if( isset( $settings['toggle'] ) ){
            $label = isset( $settings['toggle_label'] ) && !empty( $settings['toggle_label'] ) ? $settings['toggle_label'] : esc_html__( 'Click to view', 'wpt_pro' );
            echo wp_kses_post('<span class="wpt_click_to_view"><i>+</i> ' . $label . '</span>');
        }
        return;
    }
}
add_action( 'wpto_column_top', 'wpt_pro_add_toggle_content', 10, 3);

if( !function_exists( 'wpt_show_variation_table' ) ){
    
    /**
     * Show variation table on each and every variable product page
     * which will replaced the default variation dropdown select options.
     * 
     * 
     * @since 7.0.8.1
     */
    function wpt_show_variation_table(){
        
        global $product;
        //var_dump($product->get_type());
        if( $product->get_type() !== 'variable' ) return;
        
        
        $config = get_option( 'wpt_configure_options' ); 
        $enable = isset( $config['table_for_variable_product'] ) && $config['table_for_variable_product'] == 'on' ? true : false;
        //if( isset( $config['table_on_archive'] ) && $config['table_on_archive'] == 'on' )
        
        $table_id = isset( $config['variation_table_id'] ) ? $config['variation_table_id'] : false;
        $table_position = isset( $config['variation_table_position'] ) ? $config['variation_table_position'] : false;
        $table_id = apply_filters( 'wpto_variation_table_id', $table_id );
        $table_id = is_numeric( $table_id ) ? (int) $table_id : false;

        if( $table_id && $enable ){
            remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_add_to_cart', 30 );
          if ($table_position == 'woocommerce_single_product_summary') {
                add_action($table_position, 'wpt_woo_product_table_as_variation', 4);
            }else{
                add_action($table_position, 'wpt_woo_product_table_as_variation', -999);
            }
        }
        
        
    }
}

add_action( 'woocommerce_single_product_summary', 'wpt_show_variation_table', 3 );


if( !function_exists( 'wpt_woo_product_table_as_variation' ) ){
    function wpt_woo_product_table_as_variation(){

        $config = get_option( 'wpt_configure_options' ); 

        $table_id = isset( $config['variation_table_id'] ) ? $config['variation_table_id'] : false;
        $table_id = apply_filters( 'wpto_variation_table_id', $table_id );
        $table_id = is_numeric( $table_id ) ? (int) $table_id : false;
        //var_dump($table_id);
        if( $table_id ){
            add_filter( 'wpto_table_query_args', 'wpt_variation_table_args' );
            echo do_shortcode( "[Product_Table id='{$table_id}']" );
        }
    }
}

if( !function_exists( 'wpt_variation_table_args' ) ){
    
    /**
     * Args manipulation for variable product
     * 
     * @param array $args WPT default args array
     * @return array
     */
    function wpt_variation_table_args( $args ){

        $_ID = get_the_ID();
        if( empty( $_ID ) || ! is_product() ) return $args;
        $product_includes = [];
        $this_post_variable = new WC_Product_Variable( $_ID );
        $available_post_includes = $this_post_variable->get_children();

        if( isset( $available_post_includes ) && is_array($available_post_includes) && count( $available_post_includes ) > 0 ){
            foreach ($available_post_includes as $pperItem){
                $product_includes[$pperItem] = $pperItem;
            }
        }

        
        $args['post_type'] = array( 'product', 'product_variation' );
        $args['post__in'] = $product_includes;
        unset( $args['tax_query'] );
        // unset( $args['meta_query'] ); //stock kaj korchilona bole off korechi. notun somossay paile abar dekhote hobe: 8.0.8.1

        return $args;
    }
}

/****************** NEW CODE *********************/


if( !function_exists( 'wpt_quckcart_fragments_for_products' ) ){
    function wpt_quckcart_fragments_for_products( $fragments ){
        ob_start();
        $Cart = WC()->cart->cart_contents;
        $product_response = false;
        if( is_array( $Cart ) && count( $Cart ) > 0 ){
            foreach($Cart as $perItem){
                
                $pr_id = (String) $perItem['product_id'];
                $vr_id = (String) $perItem['variation_id'];
                $pr_value = (String) $perItem['quantity'];
                $product_response[$pr_id] = $pr_value;
                $product_response[$vr_id] = $pr_value;
            }
        }
        $fragments['wpt_quckcart'] = $product_response;
        
        return $fragments;
    }
}
add_filter( 'woocommerce_add_to_cart_fragments', 'wpt_quckcart_fragments_for_products', 10 );



if( !function_exists( 'wpt_quckcart_ajax_update' ) ){
    function wpt_quckcart_ajax_update(){
        global $wpdb, $woocommerce;
        
        $contents = $woocommerce->cart->get_cart();

        $product_id = !empty( $_POST['product_id'] ) ? $_POST['product_id'] : false;
        $qty_val = !empty( $_POST['qty_val'] ) ? $_POST['qty_val'] : 0;
        $quantity = is_numeric( $qty_val ) ? (int) $qty_val : $qty_val;
        if( !$product_id ){
            return;
        }

        $product = wc_get_product( $product_id );
        $stock = $product->get_stock_quantity();
        if( ! empty( $stock ) && $quantity > $stock ){
            $quantity = $stock; 
        }

        if( $product->is_sold_individually() && $quantity > 0 ){
            $quantity = 1;
        }

        
        $add_to_cart = true;
        
        foreach ( $contents as $cart_item_key => $cart_item_data ){
            

            if( $cart_item_data['product_id'] == $product_id || $cart_item_data['variation_id'] == $product_id ){
                $add_to_cart = false;

                WC()->cart->set_quantity( $cart_item_key, $quantity, true );

            }
            
        }
        if( $add_to_cart ){
            $woocommerce->cart->add_to_cart( $product_id, $quantity );
        }
        
        WC()->cart->calculate_totals();

        WC_AJAX::get_refreshed_fragments();
        die();
    }
}
add_action( 'wp_ajax_wpt_quckcart_ajax_update', 'wpt_quckcart_ajax_update' );
add_action( 'wp_ajax_nopriv_wpt_quckcart_ajax_update', 'wpt_quckcart_ajax_update' );


if( !function_exists( 'wpt_quckcart_count_info' ) ){
    function wpt_quckcart_count_info( $fragments ){
        WC()->cart->calculate_totals();
        WC_AJAX::get_refreshed_fragments();
    }
}
add_action( 'wp_ajax_wpt_quckcart_count_info', 'wpt_quckcart_count_info' );
add_action( 'wp_ajax_wpt_quckcart_count_info', 'wpt_quckcart_count_info' );

if( !function_exists( 'wpt_pro_cf_filter_show' ) ){
    
    /**
     * Displaying Custom Field Filter Showing 
     * Under Advance Search Filter
     * 
     * 
     * Create filter using custom field for Product Table
     * This feature will be found under Search & Filter tab
     * 
     * @since 7.0.9
     */
    function wpt_pro_cf_filter_show( $table_ID ){

        $meta =  get_post_meta( $table_ID, 'search_n_filter', true );
        
        if( isset( $meta['cf_search_box'] ) && $meta['cf_search_box'] == 'no' ){
            return;
        }
       
        
        
        $data = get_post_meta( $table_ID, '_cf_filter', true );
        $label_text = get_post_meta($table_ID,'_cf_filter_text_data',true );

        $choose_text = isset( $label_text['choose_text'] ) && !empty( $label_text['choose_text'] ) ? $label_text['choose_text'] : "Choose";
        $multiple_bool = isset( $label_text['choose_text'] ) && empty( $label_text['choose_text'] ) ? true : false;
        $multiple = apply_filters('wpt_custom_field_multiple', $multiple_bool, $table_ID );
        $multiple_txt = $multiple ? 'multiple' : '';
        
        $html = "";
        if( is_array( $data ) && count( $data ) > 0 ):

            $inside_html = false;

            foreach( $data as $key => $items ){

                if( ! isset( $items['values'] ) || ! isset( $items['label'] ) ) continue;

                if( is_array( $items ) && is_array( $items['values'] ) ){
                    $name = $items['label'];
                    $item_values = $items['values'];
                    $inside_html .='<div class="search_single_column cf_fielter cf_fielter_wpt">';
                    $inside_html .= '<label class="search_keyword_label">' . $name . '</label>';
                    $inside_html .='<select data-key="' . $key . '"  class="search_select cf_query cf_query_' . $multiple_txt . '" tabindex="-1" ' . $multiple_txt . '>';
                    
                    $inside_html .= ! $multiple ? '<option value="">' . $choose_text . ' ' . $name . '</option>' : '';
                    foreach( $item_values as $item ){
                        $option_label = apply_filters( 'wpt_custom_field_option_label', $item, $table_ID );
                        $inside_html .='<option value="' . $item . '">' . $option_label . '</option>';
                    }

                    $inside_html .='</select >';
                    $inside_html .= '</div>';
                }

            }
            if( $inside_html ){
                $html .= "<div id='search_box_{$table_ID}' class='wpt_search_box search_box_{$table_ID}'>";
                $html .= "<div class='wpt_filter_wrapper'>";
                $html .= $inside_html;
                $html .= '<button data-type="query" data-temp_number="' . $table_ID . '" id="wpt_query_search_button_' . $table_ID . '" class="button wpt_search_button query_button wpt_query_search_button wpt_query_search_button_' . $table_ID . '">Search</button>';
                $html .= "</div>";
                $html .= "</div>";
            }

        endif;
        echo $html;
    }

}
add_action( 'wpto_after_advance_search_box', 'wpt_pro_cf_filter_show' );


if( ! function_exists( 'wpt_por_on_sale_dropdown' ) ){
    
    /**
     * On sale dropdown or checkbox showing in frontend inside advance search box
     * 
     * @since 1.0.0
     */
    function wpt_por_on_sale_dropdown( $temp_number, $config_value ){
        $html = '';
        $meta =  get_post_meta( $temp_number, 'search_n_filter', true );

        
        $fields = $meta['fields'] ?? array();
        if( empty( $fields['on_sale'] ) ) return;
        
        $sale = isset( $_GET['sale'] ) ? sanitize_text_field( $_GET['sale'] ) : '';
        $single_keyword = $config_value['search_box_sale'] ?? __( 'Sale', 'wpt_pro' );
        
        /**
         * @Hook Filter Hook for change search box sale option
         * 
         * @since 8.0.5.0
         * @author Saiful Islam <codersaiful@gmail.com>
         */
        $on_sale_options = apply_filters( 'wpto_search_sale_option', array(
            'no_sale' => __( 'Default','wpt_pro' ),
            'sale_product' => __( 'On Sale','wpt_pro' ),
        ), $temp_number, $config_value );

        $html .= "<div class='search_single_column search_single_order'>";
        $html .= '<label class="search_keyword_label single_keyword" for="order_' . $temp_number . '">' . $single_keyword . '</label>';
        $html .= '<select data-key="on_sale" id="order_' . $temp_number . '" class="query_box_direct_value select2" >  ';
        $html .= '<option value="no_sale">'. esc_html( $on_sale_options['no_sale'] ).'</option>';
        $html .= '<option value="sale_product" '. wpt_check_sortOrder( $sale, 'sale_product' ) .'>'. esc_html( $on_sale_options['sale_product'] ) .'</option>';
        
        $html .= '</select>';

        $html .= "</div>";// End of .search_single_column
        echo $html;
    }

}
add_action( 'wpto_search_box_basics', 'wpt_por_on_sale_dropdown', 9, 2 );


if( ! function_exists( 'wpt_searchbox_extra_basic' ) ){
    
    function wpt_searchbox_extra_basic( $temp_number, $config_value, $order_by, $order ){
        $html = '';
        $meta =  get_post_meta( $temp_number, 'search_n_filter', true );
        $fields = $meta['fields'] ?? array();

        
        if( ! empty( $fields['orderby'] ) ){
        $order_by = isset( $_GET['orderby'] ) ? sanitize_text_field( $_GET['orderby'] ) : $order_by;
        $single_keyword = $config_value['search_box_orderby'] ?? '';//__( 'Order By', 'wpt_pro' ); //search_box_orderby
        
        $catalog_orderby_options = apply_filters(
			'woocommerce_catalog_orderby',
			array(
				'menu_order' => __( 'Default sorting', 'woocommerce' ),
				'popularity' => __( 'Sort by popularity', 'woocommerce' ),
				'rating'     => __( 'Sort by average rating', 'woocommerce' ),
				'date'       => __( 'Sort by latest', 'woocommerce' ),
				'price'      => __( 'Sort by price: low to high', 'woocommerce' ),
				'price-desc' => __( 'Sort by price: high to low', 'woocommerce' ),
                //New added @since 3.2.3.18.0.8.1
                // 'featured_products' => __( 'Sort by Featured Products', 'woocommerce' ),
			)
		);
        
        $html .= "<div class='search_single_column search_single_sort search_single_order_by'>";
            $html .= '<label class="search_keyword_label single_keyword" for="order_by' . $temp_number . '">' . $single_keyword . '</label>';

            $html .= '<select data-key="orderby" id="order_by_' . $temp_number . '" class="query_box_direct_value select2" >';
            

            $html .= '<option value="">'.esc_html__( 'None','wpt_pro' ).'</option>';
            // $html .= '<option value="name" '. wpt_check_sortOrder( $order_by, 'name' ) .'>'.esc_html__( 'Name','wpt_pro' ).'</option>';
            foreach ( $catalog_orderby_options as $id => $name ) :
            $html .= '<option value="' . $id . '" '. wpt_check_sortOrder( $order_by, $id ) .'>'. esc_html( $name ) .'</option>';
            endforeach;

            
            $html .= '<option value="name" '. wpt_check_sortOrder( $order_by, 'name' ) .'>'.esc_html__( 'Sort by name','wpt_pro' ).'</option>';
            // $html .= '<option value="menu_order" '. wpt_check_sortOrder( $order_by, 'menu_order' ) .'>'.esc_html__( 'Menu Order','wpt_pro' ).'</option>';
            // $html .= '<option value="type" '. wpt_check_sortOrder( $order_by, 'type' ) .'>'.esc_html__( 'Type','wpt_pro' ).'</option>';
            // $html .= '<option value="comment_count" '. wpt_check_sortOrder( $order_by, 'comment_count' ) .'>'.esc_html__( 'Reviews','wpt_pro' ).'</option>';
            $html .= '</select>';
        $html .= "</div>";
        }
        if( ! empty( $fields['order'] ) ){
        $order = isset( $_GET['order'] ) ? sanitize_text_field( $_GET['order'] ) : $order;
        $single_keyword = $config_value['search_box_order'] ?? ''; 
        $html .= "<div class='search_single_column search_single_order'>";
            $html .= '<label class="search_keyword_label single_keyword" for="order_' . $temp_number . '">' . $single_keyword . '</label>';
            $html .= '<select data-key="order" id="order_' . $temp_number . '" class="query_box_direct_value select2" >  ';
            $html .= '<option value="ASC" '. wpt_check_sortOrder( $order, 'ASC' ) .'>'.esc_html__( 'ASCENDING','wpt_pro' ).'</option>';
            $html .= '<option value="DESC" '. wpt_check_sortOrder( $order, 'DESC' ) .'>'.esc_html__( 'DESCENDING','wpt_pro' ).'</option>';
            $html .= '<option value="random" '. wpt_check_sortOrder( $order, 'random' ) .'>'.esc_html__( 'Random','wpt_pro' ).'</option>';
            $html .= '</select>';

        $html .= "</div>";// End of .search_single_column
        }
        
        echo $html;


    }
}
add_action( 'wpto_search_box_basics', 'wpt_searchbox_extra_basic', 10, 4 );


if( ! function_exists( 'wpt_sort_type_wise_adv_search' ) ){
    
    /**
     * This function has fixed shop archive default sorting issue.
     * 
     * @param type $args
     * @return array $args
     */
    function wpt_sort_type_wise_adv_search( $args ){
            
            $_orderby = $args['orderby'] ?? '';
            
            switch( $_orderby ){
                case 'price':
                        $args['orderby'] = 'meta_value_num';
                        $args['order'] = 'asc';
                        $args['meta_key'] = '_price';
                    break;
                case 'price-desc':
                        $args['orderby'] = 'meta_value_num';
                        $args['order'] = 'desc';
                        $args['meta_key'] = '_price';
                    break;
                case 'rating':
                        $args['orderby'] = 'meta_value_num';
                        $args['order'] = 'desc';
                        $args['meta_key'] = '_wc_average_rating';
                   break;
                case 'popularity':
                        $args['orderby'] = 'meta_value_num';
                        $args['order'] = 'desc';
                        $args['meta_key'] = 'total_sales';
                    break;
                case 'date':
                        $args['order'] = 'desc';
                    break;
            }
            
        return $args;
    }
}
add_filter( 'wpto_table_query_args_in_row', 'wpt_sort_type_wise_adv_search', 99 );


if( !function_exists( 'wpt_on_sale_arg_manipulation' ) ){
    /**
     * On sale manipulation for front end.
     * there will stay a dropdown or checkbox in advance search box
     * and based on on_sale product value
     * query will be regenrate
     * 
     * @since 8.0.5.0
     * 
     */
    function wpt_on_sale_arg_manipulation( $args, $directkey, $targetTableArgs, $custom_field, $texonomies ){

        $sale = $directkey['on_sale'] ?? '';
        if( empty( $sale ) ) return $args;

        $table_ID = $args['table_ID'] ?? 0;
        $basics = get_post_meta( $table_ID, 'basics', true );
        
        $post_include = isset( $basics['post_include'] ) ? $basics['post_include'] : false;
        $sale = $directkey['on_sale'] ?? '';
        if( $sale == 'sale_product' ){
            $sale_products = wc_get_product_ids_on_sale();
            $sale_products = $sale_products && is_array( $sale_products ) && $post_include && is_array( $post_include ) ? array_intersect( $post_include, $sale_products ) : $sale_products;
            $args['post__in'] = $sale_products;//var_dump(wc_get_product_ids_on_sale());
        }elseif( $sale == 'no_sale' ){
            $args['post__in'] = $post_include;
        }

        return $args;
    }
}
add_filter( 'wpto_query_arg_ajax', 'wpt_on_sale_arg_manipulation', 11, 5 );

//It's for new version of Product Table
add_filter( 'wpt_query_args', 'wpt_sale_organize' );

/**
 * Find sale product for new version of shortcode Object
 * 
 * @since 8.0.9.1
 */
function wpt_sale_organize($args){
    $on_sale = $args['on_sale'] ?? false;
    if($on_sale !== 'sale_product') return $args;
    
    $post__in = $args['post__in'] ?? [];
    $sale_product = wc_get_product_ids_on_sale();
    $args['post__in'] == ! empty( $post__in ) && is_array( $post__in ) ? array_intersect( $post__in, $sale_product ) : $sale_product;

    return $args;
}


/**
 * Track product views.
 */
function wpt_track_product_view() {
	if ( ! is_singular( 'product' ) || is_active_widget( false, false, 'woocommerce_recently_viewed_products', true ) ) {
		return;
	}

	global $post;

	if ( empty( $_COOKIE['woocommerce_recently_viewed'] ) ) { // @codingStandardsIgnoreLine.
		$viewed_products = array();
	} else {
		$viewed_products = wp_parse_id_list( (array) explode( '|', wp_unslash( $_COOKIE['woocommerce_recently_viewed'] ) ) ); // @codingStandardsIgnoreLine.
	}

	// Unset if already in viewed products list.
	$keys = array_flip( $viewed_products );

	if ( isset( $keys[ $post->ID ] ) ) {
		unset( $viewed_products[ $keys[ $post->ID ] ] );
	}

	$viewed_products[] = $post->ID;

	if ( count( $viewed_products ) > 15 ) {
		array_shift( $viewed_products );
	}

	// Store for session only.
	wc_setcookie( 'woocommerce_recently_viewed', implode( '|', $viewed_products ) );
}

add_action( 'template_redirect', 'wpt_track_product_view', 20 );


$cascade_filter = isset( $config['advance_cascade_filter'] ) ? $config['advance_cascade_filter'] : false;
if ($cascade_filter == 'on') {
    
    include WPTP_DIR_BASE . 'includes/cascade.php';
}

add_action( 'woocommerce_product_options_general_product_data', 'wpt_pro_product_order_meta');
function wpt_pro_product_order_meta(){

    $value = get_post_meta( get_the_ID(), '_product_order', true );
    woocommerce_wp_text_input( array(
        'id'      => '_product_order',
        'value'   => is_string( $value ) ? $value : '',
        'label'   => 'Product order',
        'type'      => 'number',
        'desc_tip' => true,
        'description' => 'You can set menu order to order your product table.',
    ) );

}

add_action( 'woocommerce_process_product_meta', 'wpt_pro_product_order_meta_save', 10, 2 );
function wpt_pro_product_order_meta_save( $id, $post ){
    $value = $_POST['_product_order'] ?? '';
    update_post_meta( $id, '_product_order', $value );

}

add_action( 'woocommerce_product_after_variable_attributes', 'wpt_pro_product_order_variation_meta', 10, 3 );
function wpt_pro_product_order_variation_meta( $loop, $variation_data, $variation ) {

    woocommerce_wp_text_input(
        array(
            'id'          => '_product_order[' . $variation->ID . ']',
            'label'   => 'Product order',
            'desc_tip' => true,
            'description' => 'You can set menu order to order your product table.',
            'value'       => get_post_meta( $variation->ID, '_product_order', true )
        )
    );

}

add_action( 'woocommerce_save_product_variation', 'wpt_pro_product_order_variation_meta_save', 10, 2 );

function wpt_pro_product_order_variation_meta_save( $post_id ) {

    $text_field = $_POST['_product_order'][ $post_id ];
    if ( ! empty( $text_field ) ) {
        update_post_meta( $post_id, '_product_order', esc_attr( $text_field ) );
    }
}


/**
 * Unncesarry column hide
 * $columns_array = apply_filters( 'wpto_final_column_arr', $columns_array );
 */
// function saiful_unnecessary_column_disablefunction($column_array){
//     // var_dump($column_array);
//     $unsets_arr = array(
//         'viewed',
//         'blank',
//         'product_id',
//         'serial_number',
//         'viewed',
//         'variation_name',
//         'length',
//         'width',
//         'height',
//         'weight',
//         'variations',
//         'quick_view',
//         'date',
//         'audio_player',
//         'height',
//         'height',
//     );
//     foreach( $unsets_arr as $arr ){
//         // var_dump($arr);
//         unset( $column_array[$arr] );
//     }
//     // var_dump($column_array);
//     return $column_array;
// }
// add_filter( 'wpto_default_column_arr', 'saiful_unnecessary_column_disablefunction' );
// add_filter( 'wpto_final_column_arr', 'saiful_unnecessary_column_disablefunction' );

/**
 * Pro version CSS template hanndle
 * we have a folder inside css folder
 * and template will load based on choosen template.
 *
 * @param [type] $tbl_id
 * @return void
 */
function wpt_pro_css_template_manage( $tbl_id ){
    $meta = get_post_meta( $tbl_id, 'table_style', true );
    $template = $meta['template'] ?? false;
    $template = apply_filters( 'wpto_table_template', $template, $tbl_id );
    if( $template == 'none' || $template == 'default' ) return;
    if( ! $template ) return;
    
    $template_dir = WPTP_BASE_DIR . 'assets/css/templates/'. $template . '.css';
    if( ! is_file( $template_dir ) ) return;

    $template_file = WOO_Product_Table::getPath('BASE_URL') . 'assets/css/templates/' . $template . '.css';
    wp_enqueue_style( 'wpt-template-' . $template , $template_file, array(), WOO_Product_Table::getVersion(), 'all' );
}
add_action( 'wpto_action_start_table','wpt_pro_css_template_manage', 999 );


/************ Module Layout System ***************/

if( !function_exists( 'wpt_catalog_display_options' ) ) {
    
    /**
     * Adding Product View Option for WooCommerce Shop Page
     * for 
     * Grid View Button
     * And Table View Button
     * 
     * It will work, when it will enable from Configuration page.
     * 
     * 
     * @return void It will display two view option.
     */
    function wpt_catalog_display_options() {
        
        $config = get_option( 'wpt_configure_options' ); 

        $on = $config['table_on_archive'] ?? false;
        $switche_on = $config['table_view_switcher'] ?? false;
        if( ! $switche_on ) return;

        $on = apply_filters( 'wpto_view_icon_on', $on, $config );
        if( ! $on ) return;

        $layout = apply_filters( 'wpto_default_layout', 'table' );
        $cook_name = 'wpt_shop_view';
        if( isset( $_GET['view'] ) && ! empty( $_GET['view'] ) ){
            $layout = $_GET['view'];
        }


        $grid = 'grid';
        $table = 'table';
        $extra_p = "?view=";
        if( ! empty( $_GET ) ){
            $args = array_filter( $_GET, function($item){
                return is_string( $item );
            } );
            
            if(isset($args['view'])){
                unset($args['view']);
            };

            $args_array = array_map(function($key, $val){
                return $key . '=' . $val;
            }, array_keys( $args ),$args);
      
            $arg_wlink = implode('&',$args_array);
            $grid_v = $extra_p . $grid . '&' . $arg_wlink;
            $table_v = $extra_p . $table . '&' . $arg_wlink;

        }else{
            $grid_v = $extra_p . $grid;
            $table_v = $extra_p . $table;
            
        }
        ?>
            <div class="wpt-pr-view-wrapper">
                <div class="wpt-product-view-option">
                    <a href="<?php echo esc_url( $grid_v ); ?>" class="product-view-btn grid-view  <?php echo esc_attr( $layout == 'grid' ? 'selected' : '' ); ?>" data-type="shop-grid"><i class="wpt-th-large"></i></a>
                    <a href="<?php echo esc_url( $table_v ); ?>" class="product-view-btn table-view <?php echo esc_attr( $layout == 'table' ? 'selected' : '' ); ?>" data-type="shop-table"><i class="wpt-grid"></i></a>
                </div>
            </div>
           
        <?php
    }
}
add_action( 'woocommerce_before_shop_loop', 'wpt_catalog_display_options', -9999 );

add_filter( 'wpto_default_archive_table_id', 'wpt_grid_table_view_by_id' );
function wpt_grid_table_view_by_id( $table_id ){
    $view = $_GET['view'] ?? '';
    if( $view == 'grid' ){
        return 0;
    }
    return $table_id;
}